//-----------------------------LICENSE NOTICE------------------------------------
//  GNU GENERAL PUBLIC LICENSE - Version 3, 29 June 2007
//  For more info, read LICENSE.txt on source's root directory
//------------------------------------------------------------------------------

#include <cpctelera.h>
#include "game.h"
#include "sprites.h"
#include "enemigo.h"
#include "menu.h"

#define VMEM (u8*)0xC000
#define MW 20 //mapa width
#define MH 11 //mapa height
#define TW 4 //tile width
#define TH 16 //tile height
#define PW 4 //personaje width
#define PH 8 //personaje height


const u8 *scene[MH];


// Mapa con 1 en los muros, 2 en las hogueras, 3 en la puerta y 4 en la llave
const u8 mapa1[MH][MW] ={{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
                         {1, 0, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 0, 0, 0, 0, 1},
                         {1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
                         {1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
                         {1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
                         {1, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
                         {1, 0, 1, 4, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 3},
                         {1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}};


const u8 mapa2[MH][MW] ={{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 4, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 1},
                         {1, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 2, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
                         {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}};   

const u8 mapa3[MH][MW] ={{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 1},
                         {1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1},
                         {1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 4, 1},
                         {1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 1},
                         {1, 1, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}};   

const u8 mapa4[MH][MW] ={{1, 1, 1, 1, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
                         {1, 0, 0, 1, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 1, 2, 0, 4, 1},
                         {1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
                         {1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
                         {1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
                         {1, 0, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 2, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 2, 1, 0, 0, 0, 0, 0, 2, 0, 0, 0, 1},
                         {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}}; 

const u8 mapa5[MH][MW] ={{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
                         {1, 0, 1, 1, 0, 0, 0, 0, 0, 2, 0, 0, 0, 1, 0, 1, 0, 0, 4, 1},
                         {1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
                         {1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
                         {1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1},
                         {1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 2, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1},
                         {1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 1, 0, 0, 0, 0, 0, 0, 1},
                         {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 1}};  

const u8 mapa6[MH][MW] ={{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
                         {1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1},
                         {1, 0, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1},
                         {1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1},
                         {1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 0, 0, 0, 0, 0, 0, 0, 1},
                         {1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 4, 1},
                         {1, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}};                                                             

//Dibujar mapa
void drawMap(u8 n){
   u8 posX=0,posY =0,x=0, num;
   u8* memptr;

	for(x=0;x<MH;x++){
		switch(n){
			case 1:
				scene[x] = mapa1[x];
				break;
			case 2:
				scene[x] = mapa2[x];
				break;
			case 3:
				scene[x] = mapa3[x];
				break;
			case 4:
				scene[x] = mapa4[x];
				break;
			case 5:
				scene[x] = mapa5[x];
				break;
			case 6:
				scene[x] = mapa6[x];
				break;
			
		}	  
	}

	for(posY=0; posY<MH;posY++){
		for(posX=0; posX<MW;posX++){
			num = scene[posY][posX];
			switch(num){
				case 1:
					memptr = cpct_getScreenPtr(VMEM, posX*TW, posY*TH); 
					cpct_drawTileAligned4x8(muro1,memptr); 
					memptr = cpct_getScreenPtr(VMEM, posX*TW, posY*TH + 8); 
					cpct_drawTileAligned4x8(muro2,memptr); 
					break;
				case 2:
					memptr = cpct_getScreenPtr(VMEM, posX*TW, (posY*TH)+3); 
					cpct_drawSpriteMasked(hoguera,memptr,4,10);
					break;
				case 3:
					memptr = cpct_getScreenPtr(VMEM, posX*TW, posY*TH); 
					cpct_drawSpriteMasked(puerta,memptr,TW,TH);
					break;
				case 4:
					memptr = cpct_getScreenPtr(VMEM, posX*TW, posY*TH); 
					cpct_drawSpriteMasked(sprite_llave,memptr,4,5);
			}
		}
	} 
}

void dibujarVida(){
  u8* memptr;
  u8 x;

  //Dibujar vida
 for(x=0; x<3; x++){
    memptr = cpct_getScreenPtr(VMEM, 10+(x*5), 185); 
    cpct_drawSolidBox(memptr, cpct_px2byteM0(0, 0), 4, 8);
    cpct_drawSpriteMasked(corazon_lleno,memptr,4,8); 
  }
}

void borrarLlave(){
  u8 posX=0,posY =0;
  u8* memptr;

  for(posY=0; posY<MH;posY++){
    for(posX=0; posX<MW;posX++){
      //Borrar llave
       if(scene[posY][posX] == 4){
          memptr = cpct_getScreenPtr(VMEM, posX*TW, posY*TH); 
          cpct_drawSolidBox(memptr, cpct_px2byteM0(0, 0), 4, 5);
       }
    }
  }
}

//Modificar los corazones que muestran la vida: se le pasa la vida que tenga 0, 1, 2 o 3
void cambiarVida(u8 vida){
  u8* memptr;
  u8 x;

  for(x=3; x>vida; x--){
      memptr = cpct_getScreenPtr(VMEM, 10+((x-1)*5), 185); 
      cpct_drawSolidBox(memptr, cpct_px2byteM0(0, 0), 4, 8);
      cpct_drawSpriteMasked(corazon_vacio,memptr,4,8); 
  }
}

//Juego
u8 game(){
  TPersonaje p = {4, 16, 0, 0, 3, pj_abajo}; //Posición 5, 0, antorcha apagada, sin llave y vida entera
  TEnemigo enemigos[3] = {{24, 150, 20, 150, 24, 150, 0, 0, 1}, {68, 112, 68, 96, 68, 112, 1, 0, 1}, {12, 150, 8, 150, 12, 150, 0, 0, 1}};
  u8* memptr;
  u8 posXant = p.x, posYant = p.y, posX, posY, pasos = 0, nivel = 1, n, vidaActual,
	  dir = 2, //0.derecha 1.izquierda 2. abajo 3.arriba
	  pasos_rest1 = 0, pasos_rest2 = 0, pasosAnt, mov = 0, movEn = 0, dibPasos = 0, iniciarP = 0;
  u8 *spritePasos1 = num0, *spritePasos2 = num0;
  

  //Borrar pantalla
  cpct_clearScreen(0);

  drawMap(nivel); //empieza en nivel 1
  dibujarVida();

    
  // Repetir para siempre
  while (1){

    //Esperar al retrazado
    cpct_waitVSYNC();

    borrarPersonaje(posXant, posYant);
    //Dibujar a personaje
    dibujarPersonaje(p.sprite, p.x, p.y);

	
	//switch de borrado y dibujado de enemigos
    //Enemigo
	
	switch(nivel){
		case 1:
	
				for(n=0; n<2; n++){
					borrarEnemigo(enemigos[n].xAnt, enemigos[n].yAnt);

					pintarEnemigo(enemigos[n].x, enemigos[n].y, enemigos[n].dir);
				}
				break;
		case 2:
			

			for(n=0; n<2; n++){
				borrarEnemigo(enemigos[n].xAnt, enemigos[n].yAnt);

				pintarEnemigo(enemigos[n].x, enemigos[n].y, enemigos[n].dir);
			}
			break;
		case 3:
		

			for(n=0; n<3; n++){
				borrarEnemigo(enemigos[n].xAnt, enemigos[n].yAnt);

				pintarEnemigo(enemigos[n].x, enemigos[n].y, enemigos[n].dir);
			}
			break;
		case 4:
		

			for(n=0; n<3; n++){
				borrarEnemigo(enemigos[n].xAnt, enemigos[n].yAnt);

				pintarEnemigo(enemigos[n].x, enemigos[n].y, enemigos[n].dir);
			}
			break;
		case 5:
			for(n=0; n<3; n++){
				borrarEnemigo(enemigos[n].xAnt, enemigos[n].yAnt);

				pintarEnemigo(enemigos[n].x, enemigos[n].y, enemigos[n].dir);
			}
			break;
		case 6:
			for(n=0; n<3; n++){
				borrarEnemigo(enemigos[n].xAnt, enemigos[n].yAnt);

				pintarEnemigo(enemigos[n].x, enemigos[n].y, enemigos[n].dir);
			}
			break;
	}

     

    //Cambiar aspecto de los corazones
	vidaActual = p.vida;
	switch(vidaActual){
		case 0:
			 cambiarVida(0);
			 return 1;
			 break;
		case 1:
			cambiarVida(1);
			break;
		case 2:
			cambiarVida(2);
			break;
		default:
			break;
	}

    if(p.antorcha == 1 && dibPasos == 4){

      //Mostrar pasos restantes con la antorcha
      memptr = cpct_getScreenPtr(VMEM, 46, 185); 
      cpct_drawSpriteMasked(antorcha,memptr,3,8);

      memptr = cpct_getScreenPtr(VMEM, 50, 185); 
      cpct_drawSolidBox(memptr, 0, 15, 10); 

      if(pasos_rest1 != 0)
        cpct_drawSprite(spritePasos1,memptr,4,8);

      memptr = cpct_getScreenPtr(VMEM, 55, 185); 
      cpct_drawSprite(spritePasos2,memptr,4,8); 

      dibPasos == 0;

    }else if(p.antorcha == 1 && dibPasos < 4){
      dibPasos++;
    }
    
    //**************CÁLCULOS****************

    posXant = p.x;
    posYant = p.y;
    pasosAnt = pasos;

    //Comprobar el teclado
    cpct_scanKeyboard_f();

    //DERECHA
    if(cpct_isKeyPressed(Key_D) || cpct_isKeyPressed(Key_CursorRight) && p.x < (80-PW)){ //Si la tecla cursor a la derecha está pulsada y mi x no está al final de la pantalla (80-4 del personaje = 76)
        
        if(mov == 1){//mov evita que el movimiento se haga muy rapidamente
          p.x += 1; //Movemos la coordenada x un byte a la derecha (2 píxeles)
          pasos += 1;
          dir=0;//derecha
          mov = 0;
        }else{
          mov++;
        }
    //IZQUIERDA
    }else if(cpct_isKeyPressed(Key_A) || cpct_isKeyPressed(Key_CursorLeft) && p.x > 0){ //Si la tecla cursor a la izquierda está pulsada y mi x no está al principio de la pantalla (0)
        
		if(mov == 1){
			p.x -= 1; //Movemos la coordenada x un byte a la derecha (2 píxeles)
			pasos += 1;
			dir=1;//izda
			mov = 0;
        }else{
          mov++;
        }
        
    //ARRIBA
    }else if(cpct_isKeyPressed(Key_W) || cpct_isKeyPressed(Key_CursorUp) && p.y > 0){ 
      
         p.y -= 2; 
         pasos += 1;
		      dir=3;//arriba
    
    //ABAJO
    }else if(cpct_isKeyPressed(Key_S) || cpct_isKeyPressed(Key_CursorDown) && p.y < (200-PH)){ 
      
         p.y += 2; 
         pasos += 1;
		      dir=2;//abajo
    
    }else if(cpct_isKeyPressed(Key_Esc)){
      return 0; 
    }

    posX = p.x;
    posY = p.y;
   

    ///////////////////////Cálculo de las colisiones del personaje con todo 
  if((scene[(posY)/TH][(posX)/TW] != 0) ||(scene[(posY)/TH][(posX+PW-1)/TW] != 0) 
      || (scene[(posY+PH-2)/TH][(posX+PW-1)/TW] != 0) || (scene[(posY+PH-2)/TH][(posX)/TW] != 0)){
     
       //colision con muro
      if((scene[(posY)/TH][(posX)/TW]==1) ||(scene[(posY)/TH][(posX+PW-1)/TW]==1) 
      || (scene[(posY+PH-2)/TH][(posX+PW-1)/TW]==1) || (scene[(posY+PH-2)/TH][(posX)/TW]==1)){

          p.x = posXant;
          p.y = posYant;
          pasos = pasosAnt;

      }else 
        //Cálculo de las colisiones con hogueras -> encender antorcha
        if((scene[(posY)/TH][(posX)/TW]==2) ||(scene[(posY)/TH][(posX+PW-1)/TW]==2) 
          || (scene[(posY+PH-2)/TH][(posX+PW-1)/TW]==2) || (scene[(posY+PH-2)/TH][(posX)/TW]==2)){
          p.x = posXant;
          p.y = posYant;
          //Encender antorcha
          p.antorcha = 1;
          pasos = 0;
      }else 
        //Cálculo coger llave
        if(p.llave == 0 && ((scene[(posY)/TH][(posX)/TW]==4) ||(scene[(posY)/TH][(posX+PW-1)/TW]==4) 
          || (scene[(posY+PH-2)/TH][(posX+PW-1)/TW]==4) || (scene[(posY+PH-2)/TH][(posX)/TW]==4))){

          //Coger llave
          p.llave = 1;
          //Borrar llave del mapa
          borrarLlave();
          //Dibujar llave en el HUD
          memptr = cpct_getScreenPtr(VMEM, 70, 185); 
          cpct_drawSpriteMasked(sprite_llave,memptr,4,5);

      }else
       //Cálculo de atravesar puerta
        if((scene[(posY)/TH][(posX)/TW]==3) ||(scene[(posY)/TH][(posX+PW-1)/TW]==3) 
          || (scene[(posY+PH-2)/TH][(posX+PW-1)/TW]==3) || (scene[(posY+PH-2)/TH][(posX)/TW]==3)){
          
          //Comprobar que el personaje tenga la llave
          if(p.llave == 1){
           
            //Cambiar nivel
            if(nivel < 6)
              nivel++;
            else
              return 2;

            //Cambiar enemigos del nivel
            switch(nivel){
              case 2:
             
                  enemigos[0].xIn = 12;
                  enemigos[0].yIn = 102;
                  enemigos[0].xAnt = 12;
                  enemigos[0].yAnt = 102;
                  enemigos[0].x = 12;
                  enemigos[0].y = 102;
                  enemigos[0].tipo = 1;
                  enemigos[0].estado = 0;
                  enemigos[0].dir = 1;

                  enemigos[1].xIn = 48;
                  enemigos[1].yIn = 48;
                  enemigos[1].xAnt = 48;
                  enemigos[1].yAnt = 48;
                  enemigos[1].x = 48;
                  enemigos[1].y = 48;
                  enemigos[1].tipo = 1;
                  enemigos[1].estado = 0;
                  enemigos[1].dir = 1;

                  break;
                case 3:

                  enemigos[0].xIn = 52;
                  enemigos[0].yIn = 56;
                  enemigos[0].xAnt = 52;
                  enemigos[0].yAnt = 56;
                  enemigos[0].x = 52;
                  enemigos[0].y = 56;
                  enemigos[0].tipo = 1;
                  enemigos[0].estado = 0;
                  enemigos[0].dir = 1;

                  enemigos[1].xIn = 64;
                  enemigos[1].yIn = 120;
                  enemigos[1].xAnt = 64;
                  enemigos[1].yAnt = 120;
                  enemigos[1].x = 64;
                  enemigos[1].y = 120;
                  enemigos[1].tipo = 0;
                  enemigos[1].estado = 0;
                  enemigos[1].dir = 1;

                  break;

                case 4:

                  enemigos[0].xIn = 24;
                  enemigos[0].yIn = 128;
                  enemigos[0].xAnt = 24;
                  enemigos[0].yAnt = 128;
                  enemigos[0].x = 24;
                  enemigos[0].y = 128;
                  enemigos[0].tipo = 0;
                  enemigos[0].estado = 0;
                  enemigos[0].dir = 1;

                  enemigos[1].xIn = 48;
                  enemigos[1].yIn = 48;
                  enemigos[1].xAnt = 48;
                  enemigos[1].yAnt = 48;
                  enemigos[1].x = 48;
                  enemigos[1].y = 48;
                  enemigos[1].tipo = 0;
                  enemigos[1].estado = 0;
                  enemigos[1].dir = 1;

                  enemigos[2].xIn = 68;
                  enemigos[2].yIn = 64;
                  enemigos[2].xAnt = 68;
                  enemigos[2].yAnt = 64;
                  enemigos[2].x = 68;
                  enemigos[2].y = 64;
                  enemigos[2].tipo = 1;
                  enemigos[2].estado = 0;
                  enemigos[2].dir = 1;

                  break;
				  
				case 5:
				  enemigos[0].xIn = 96;
                  enemigos[0].yIn = 48;
                  enemigos[0].xAnt = 96;
                  enemigos[0].yAnt = 48;
                  enemigos[0].x = 96;
                  enemigos[0].y = 48;
                  enemigos[0].tipo = 0;
                  enemigos[0].estado = 0;
                  enemigos[0].dir = 1;
				  
				  enemigos[1].xIn = 68;
                  enemigos[1].yIn = 64;
                  enemigos[1].xAnt = 68;
                  enemigos[1].yAnt = 64;
                  enemigos[1].x = 68;
                  enemigos[1].y = 64;
                  enemigos[1].tipo = 1;
                  enemigos[1].estado = 0;
                  enemigos[1].dir = 1;

                  enemigos[2].xIn = 36;
                  enemigos[2].yIn = 128;
                  enemigos[2].xAnt = 36;
                  enemigos[2].yAnt = 128;
                  enemigos[2].x = 36;
                  enemigos[2].y = 128;
                  enemigos[2].tipo = 0;
                  enemigos[2].estado = 0;
                  enemigos[2].dir = 1;
					break;
					
				case 6:
				
				  enemigos[0].xIn = 32;
                  enemigos[0].yIn = 80;
                  enemigos[0].xAnt = 32;
                  enemigos[0].yAnt = 80;
                  enemigos[0].x = 32;
                  enemigos[0].y = 80;
                  enemigos[0].tipo = 0;
                  enemigos[0].estado = 0;
                  enemigos[0].dir = 1;
				  
				  enemigos[1].xIn = 28;
                  enemigos[1].yIn = 144;
                  enemigos[1].xAnt = 28;
                  enemigos[1].yAnt = 144;
                  enemigos[1].x = 28;
                  enemigos[1].y = 144;
                  enemigos[1].tipo = 0;
                  enemigos[1].estado = 0;
                  enemigos[1].dir = 0;
				  
				  enemigos[2].xIn = 52;
                  enemigos[2].yIn = 96;
                  enemigos[2].xAnt = 52;
                  enemigos[2].yAnt = 96;
                  enemigos[2].x = 52;
                  enemigos[2].y = 96;
                  enemigos[2].tipo = 1;
                  enemigos[2].estado = 0;
                  enemigos[2].dir = 1;
					break;
            } //fin switch cambio de nivel

            //Inicializar personaje
            p.x = 4;
            p.y = 16;
            p.antorcha = 0;
            p.llave = 0;
            pasos = 0;
            dir=2;
            p.vida = 3;

            cpct_clearScreen(0);
            //Dibujar nivel
            drawMap(nivel);

            dibujarVida();

            //Colisionar si no lleva la llave  
          }else{
            p.x = posXant;
            p.y = posYant;
          }
      }
  }
  ////////////fin de calculo de colisiones

   // Apagar antorcha cada ciertos pasos

   if(pasos >= 60){
      p.antorcha = 0;
      pasos = 0;

      //Borrar pasos del HUD
      memptr = cpct_getScreenPtr(VMEM, 46, 185); 
      cpct_drawSolidBox(memptr, 0, 14, 10);
   }

   if(p.antorcha == 1){
      pasos_rest1 = (60-pasos)/10;
      pasos_rest2 = (60-pasos)-(pasos_rest1*10);


      switch(pasos_rest1){
        case 1: spritePasos1 = num1;
                break;
        case 2: spritePasos1 = num2;
                break;               
        case 3: spritePasos1 = num3;
                break;   
        case 4: spritePasos1 = num4;
                break;
        case 5: spritePasos1 = num5;
                break;
        case 6: spritePasos1 = num6;
                break;
        case 7: spritePasos1 = num7;
                break;
        case 8: spritePasos1 = num8;
                break;
        case 9: spritePasos1 = num9;
                break;
      }

      switch(pasos_rest2){
        case 0: spritePasos2 = num0;
                break;
        case 1: spritePasos2 = num1;
                break;
        case 2: spritePasos2 = num2;
                break;               
        case 3: spritePasos2 = num3;
                break;   
        case 4: spritePasos2 = num4;
                break;
        case 5: spritePasos2 = num5;
                break;
        case 6: spritePasos2 = num6;
                break;
        case 7: spritePasos2 = num7;
                break;
        case 8: spritePasos2 = num8;
                break;
        case 9: spritePasos2 = num9;
                break;
      }
   }
  

   //Enemigo
   if(movEn < 1){
      movEn ++;
   }else{
  	switch(nivel){
  		case 1:
  			for(n=0; n<2; n++){
  			  enemigos[n].xAnt = enemigos[n].x;
  			  enemigos[n].yAnt = enemigos[n].y;

  			  comprobarRango(&enemigos[n], scene, (p.x+(PW/2)) / TW, (p.y+(PH/2)) / TH); 

  			  if(enemigos[n].estado == 0)
  				patrullar(&enemigos[n], scene);
  			  else{
  				if(p.antorcha == 1){
  				  huir(&enemigos[n], p.x, p.y, scene);
  				}else{
  				  iniciarP = atacar(&enemigos[n], &p, scene);

            //Volver a la posicion inicial si pierdes una vida
            if(iniciarP == 1){
              p.x = 4;
              p.y = 16;
              p.antorcha = 0;
              p.sprite = pj_abajo;
              dir = 2;
              iniciarP = 0;
            }

  				}
  			  }
  			}
  			break;
  		case 2:
  			

  			for(n=0; n<2; n++){

  				enemigos[n].xAnt = enemigos[n].x;
  				enemigos[n].yAnt = enemigos[n].y;

  				comprobarRango(&enemigos[n], scene, (p.x+(PW/2)) / TW, (p.y+(PH/2)) / TH);


  				if(enemigos[n].estado == 0)
  					patrullar(&enemigos[n], scene);
  				else{
  					if(p.antorcha == 1){
  						huir(&enemigos[n], p.x, p.y, scene);
  					}else{
  						iniciarP = atacar(&enemigos[n], &p, scene);

              //Volver a la posicion inicial si pierdes una vida
              if(iniciarP == 1){
                p.x = 4;
                p.y = 16;
                p.antorcha = 0;
                p.sprite = pj_abajo;
                dir = 2;
                iniciarP = 0;
              }

  					}
  				}
  			}
  		
  			break;
  		case 3:

  			for(n=0; n<3; n++){

  			  enemigos[n].xAnt = enemigos[n].x;
  			  enemigos[n].yAnt = enemigos[n].y;

  			  comprobarRango(&enemigos[n], scene, (p.x+(PW/2)) / TW, (p.y+(PH/2)) / TH);
  				
  			  if(enemigos[n].estado == 0)
  				patrullar(&enemigos[n], scene);
  			  else{
  				if(p.antorcha == 1){
  				  huir(&enemigos[n], p.x, p.y, scene);
  				}else{
  				  iniciarP = atacar(&enemigos[n], &p, scene);

            //Volver a la posicion inicial si pierdes una vida
            if(iniciarP == 1){
              p.x = 4;
              p.y = 16;
              p.antorcha = 0;
              p.sprite = pj_abajo;
              dir = 2;
              iniciarP = 0;
            }
  				}
  			  }

  			 }
  			break;
  		case 4:

  			for(n=0; n<3; n++){

  			  enemigos[n].xAnt = enemigos[n].x;
  			  enemigos[n].yAnt = enemigos[n].y;

  			  comprobarRango(&enemigos[n], scene, (p.x+(PW/2)) / TW, (p.y+(PH/2)) / TH);
  				

  			  if(enemigos[n].estado == 0)
  				patrullar(&enemigos[n], scene);
  			  else{
  				if(p.antorcha == 1){
  				  huir(&enemigos[n], p.x, p.y, scene);
  				}else{
  				  iniciarP = atacar(&enemigos[n], &p, scene);

            //Volver a la posicion inicial si pierdes una vida
            if(iniciarP == 1){
              p.x = 4;
              p.y = 16;
              p.antorcha = 0;
              p.sprite = pj_abajo;
              dir = 2;
              iniciarP = 0;
            }

  				}
  			  }

  			}
  		  
  			break;
			
			case 5:

  			for(n=0; n<3; n++){

  			  enemigos[n].xAnt = enemigos[n].x;
  			  enemigos[n].yAnt = enemigos[n].y;

  			  comprobarRango(&enemigos[n], scene, (p.x+(PW/2)) / TW, (p.y+(PH/2)) / TH);
  				
  			  if(enemigos[n].estado == 0)
  				patrullar(&enemigos[n], scene);
  			  else{
  				if(p.antorcha == 1){
  				  huir(&enemigos[n], p.x, p.y, scene);
  				}else{
  				  iniciarP = atacar(&enemigos[n], &p, scene);

            //Volver a la posicion inicial si pierdes una vida
            if(iniciarP == 1){
              p.x = 4;
              p.y = 16;
              p.antorcha = 0;
              p.sprite = pj_abajo;
              dir = 2;
              iniciarP = 0;
            }
  				}
  			  }

  			 }
  			break;
			
			case 6:

  			for(n=0; n<3; n++){

  			  enemigos[n].xAnt = enemigos[n].x;
  			  enemigos[n].yAnt = enemigos[n].y;

  			  comprobarRango(&enemigos[n], scene, (p.x+(PW/2)) / TW, (p.y+(PH/2)) / TH);
  				

  			  if(enemigos[n].estado == 0)
  				patrullar(&enemigos[n], scene);
  			  else{
  				if(p.antorcha == 1){
  				  huir(&enemigos[n], p.x, p.y, scene);
  				}else{
  				  iniciarP = atacar(&enemigos[n], &p, scene);

            //Volver a la posicion inicial si pierdes una vida
            if(iniciarP == 1){
              p.x = 4;
              p.y = 16;
              p.antorcha = 0;
              p.sprite = pj_abajo;
              dir = 2;
              iniciarP = 0;
            }

  				}
  			  }

  			}
  		  
  			break;
  	}



    movEn = 0;
    p.sprite = cambiarSprites(p.sprite, p.antorcha, dir);
	}  
}/////end while bucle del juego
}

u8* cambiarSprites(u8 *sprite, u8 antorcha, u8 dir){
  u8 *s = sprite;

  // Personaje con antorcha apagada
  if(antorcha == 0){

    switch(dir){
      case 0:
         s = pj_dcha;
         break;
      case 1:
        s = pj_izda;
        break;
      case 2:
        s = pj_abajo;
        break;
      case 3:
        s = pj_arriba;
        break;
    }

  // Personaje con antorcha encendida  
  }else{
    switch(dir){
      case 0:
         s = pj_dcha_bf;
         break;
      case 1:
        s = pj_izda_bf;
        break;
      case 2:
        s = pj_abajo_bf;
        break;
      case 3:
        s = pj_arriba_bf;
        break;
    }
  }

  return s;
}

void dibujarPersonaje(u8 *sprite, u8 x, u8 y){
   u8 *memptr;
   u8 *s = sprite;

  //Dibujar al personaje
  memptr = cpct_getScreenPtr(VMEM, x, y);
  cpct_drawSpriteMasked(s, memptr, PW, PH);
}

void borrarPersonaje(u8 x, u8 y){
  u8 *memptr;

  //Borrar al personaje
  memptr = cpct_getScreenPtr(VMEM, x, y); //Las posiciones x e y están en bytes, no en píxeles, y en horizontal van de dos en dos píxeles
  cpct_drawSolidBox(memptr, 0, PW, PH); 
}



